/**
 * This file demonstrates the ability to command a group.
 */

#include <stdio.h>

#include "hebi.h"
#include "util/lookup_helpers.c"
#include "util/hebi_util.h"

int main(int argc, char* argv[])
{
  // Try and get the requested group.
  HebiGroupPtr group = getGroupFromArgs(argc, argv);
  if (group == NULL)
  {
    hebiCleanup();
    return -1;
  }

  int numModules = hebiGroupGetSize(group);

  // Create a command object; this can be sent to the group
  HebiGroupCommandPtr command = hebiGroupCommandCreate(numModules);
  if (command == NULL)
  {
    printf("Could not create command object!\n");
    hebiGroupRelease(group);
    hebiCleanup();
    return -1;
  }

  // Send commands to the group in a loop.
  // Note that these packets may be dropped if network traffic is too high, so
  // be sure to close a feedback loop at the high level!
  float period = 0.5f;
  float t;
  for (t = 0.0f; t < 10.0f; t += period)
  {
    // Set the actuator command position field for all the modules
    int module_index;
    for (module_index = 0; module_index < numModules; module_index++)
    {
      // We get the command for this particular module
      HebiCommandPtr module_cmd = hebiGroupCommandGetModuleCommand(command, module_index);
      // Set the position for this module, offsetting each with respect to other
      // modules.
      int64_t pos = 0;
      float offset = (float)sin(t * 0.5f + module_index * 0.25f);
      hebiCommandSetHighResAngle(module_cmd, HebiCommandHighResAnglePosition, &pos, &offset);
    }

    hebiGroupSendCommand(group, command);
    hebi_sleep_ms(period * 1000);
  }

  // For critical packets, we can verify that they were sent by requesting
  // confirmation from the group.  If the acknowledgement function returns
  // 'true', it got positive confirmation. If it returns 'false', EITHER:
  // - the sent command was dropped
  // - the sent command was received by the group, but its response was either
  //   dropped or the timeout period expired before receipt of the group.
  // Again, a high-level process should intelligently handle these conditions!
  // Note that this is a blocking call, and so for high-frequency applications,
  // SendCommand should be used instead.
  period = 0.5f;
  long timeout_ms = 100;
  for (t = 0.0f; t < 10.0f; t += period)
  {
    // Set the actuator command position field for all the modules
    int module_index;
    for (module_index = 0; module_index < numModules; module_index++)
    {
      // We get the command for this particular module
      HebiCommandPtr module_cmd = hebiGroupCommandGetModuleCommand(command, module_index);
      // Set the position for this module, offsetting each with respect to other
      // modules.
      int64_t pos = 0;
      float offset = (float)sin(t * 0.5f + module_index * 0.25f);
      hebiCommandSetHighResAngle(module_cmd, HebiCommandHighResAnglePosition, &pos, &offset);
    }

    if (hebiGroupSendCommandWithAcknowledgement(group, command, timeout_ms) == 0)
    {
      printf("Got acknowledgement.\n");
    }
    else
    {
      printf("Did not receive acknowledgement!\n");
    }
    hebi_sleep_ms(period * 1000);
  }

  // We must release the resources back to the library when done!
  hebiGroupCommandRelease(command);
  hebiGroupRelease(group);
  hebiCleanup();
  return 0;
}
