/**
 * This file demonstrates the ability to get feedback from a group.
 */

#include <stdio.h>

#include "hebi.h"
#include "util/feedback_print_helpers.c"
#include "util/lookup_helpers.c"
#include "util/hebi_util.h"

// This is our callback function for the async feedback loop:
// Note: the user data function can be used, for example, to store a reference
// to something in the main function.  Here, we use it to update a count of
// total messages. In C++ code, this could be a pointer to the class so this can
// effectively be treated like a member function.
void feedback_callback(HebiGroupFeedbackPtr feedback, void * user_data)
{
  // Print out position of first module:
  HebiFeedbackPtr first_module_feedback =
    hebiGroupFeedbackGetModuleFeedback(feedback, 0);
  int64_t revs; float offset;
  if (hebiFeedbackGetHighResAngle(first_module_feedback, HebiFeedbackHighResAnglePosition, &revs, &offset) == HebiStatusSuccess)
  {
    printf("%" PRId64 " + %f\n", revs, offset);
  }
  else
  {
    printf("no position feedback!\n");
  }

  // Increment feedback count:
  int* feedback_count = (int*)user_data;
  (*feedback_count)++;
}

int main(int argc, char* argv[])
{
  // Try and get the requested group.
  HebiGroupPtr group = getGroupFromArgs(argc, argv);
  if (group == NULL)
  {
    hebiCleanup();
    return -1;
  }

  // Used to demonstrate the 'user_data' field; this can be a more complex data
  // structure that you want to be accessible from the callback. Remember to use
  // good multithreading practice regarding variable acces here, as the callback
  // will be from a different thread.
  int total_feedback_calls = 0;
  hebiGroupRegisterFeedbackHandler(group, feedback_callback, &total_feedback_calls);

  // Start 200Hz feedback loop.
  printf("Starting asynchronous feedback callbacks\n");
  hebiGroupSetFeedbackFrequencyHz(group, 200);
  // Wait 10 seconds. This should result in about 2,000 callbacks.
  int wait_period_s = 10;
  hebi_sleep_s(wait_period_s);
  // Stop the feedback loop, and unrelease our callback:
  hebiGroupSetFeedbackFrequencyHz(group, 0);
  hebiGroupClearFeedbackHandlers(group);

  printf("Feedback callback thread stopped; %d callbacks were made.\n", total_feedback_calls);

  // We must release the resources back to the library when done!
  hebiGroupRelease(group);
  hebiCleanup();
  return 0;
}
