/**
 * This file demonstrates the group lookup capabilities of the API. The lookup
 * allows you to create groups of modules based on which ones are available on
 * the network.
 */

#include <stdio.h>
#include "hebi.h"
#include "util/hebi_util.h"

////////////////////////////////////////////////////////////////////////////////
// This is #defined so that the size of the arrays below can be stack allocated;
// using a "const int" for this does not work in MSVC.
////////////////////////////////////////////////////////////////////////////////
#define NAME_FAMILY_BUFFER_LENGTH 100

void printLookupTable(HebiLookupPtr lookup)
{
  // Iterate through the items discovered by the lookup so far.
  HebiLookupEntryListPtr entry_list = hebiCreateLookupEntryList(lookup);
  int i = 0;
  int num_entries = hebiLookupEntryListGetSize(entry_list);
  // Buffers to store name/family of each item; this example only supports names
  // with less than 100 characters.
  char name[NAME_FAMILY_BUFFER_LENGTH];
  char family[NAME_FAMILY_BUFFER_LENGTH];
  size_t buf_size = NAME_FAMILY_BUFFER_LENGTH;
  for (i = 0; i < num_entries; ++i)
  {
    if (hebiLookupEntryListGetName(entry_list, i, name, &buf_size) == HebiStatusSuccess)
      printf("Name: %s\n", name);
    else
      printf("Name: too large for buffer");
    if (hebiLookupEntryListGetFamily(entry_list, i, family, &buf_size) == HebiStatusSuccess)
      printf("Family: %s\n", family);
    else
      printf("Family: too large for buffer");
  }
  hebiLookupEntryListRelease(entry_list);
  printf("\n");
}

/**
 * Helper function to create mac address
 */
HebiMacAddress hebiMacAddressFromBytes(
  uint8_t a, uint8_t b, uint8_t c, uint8_t d, uint8_t e, uint8_t f)
{
  HebiMacAddress tmp;
  tmp.bytes_[0] = a;
  tmp.bytes_[1] = b;
  tmp.bytes_[2] = c;
  tmp.bytes_[3] = d;
  tmp.bytes_[4] = e;
  tmp.bytes_[5] = f;
  return tmp;
}

/**
 * Simple helper function to print out if the group is valid, and release any
 * reference to it.
 */
void checkAndReleaseGroup(HebiGroupPtr group)
{
  if (group == NULL)
  {
    printf("Group not found on network.\n\n");
    return;
  }

  printf("Found group with %zu modules.\n\n", hebiGroupGetSize(group));

  // Be sure the group is released after you are done with it.
  hebiGroupRelease(group);
}

/**
 * Attempts to look for several groups on the network (hardcoded addresses below),
 * and retrieve references to them.
 */
int main()
{
  // Create the lookup object
  HebiLookupPtr lookup = hebiLookupCreate();
  if (lookup == NULL)
  {
    printf("createLookup() returned NULL.\n\n");
    hebiCleanup();
    return 1;
  }

  // Wait for the module list to populate, and print out its contents
  hebi_sleep_s(2);
  printLookupTable(lookup);
  printf("\n");

  // Define module names/addresses:
  const HebiMacAddress* macs[2];
  int num_macs = sizeof(macs) / sizeof(macs[0]);
  HebiMacAddress addr_1 = hebiMacAddressFromBytes(0x00, 0x1e, 0xc0, 0x8d, 0xe0, 0x5c);
  macs[0] = &addr_1;
  HebiMacAddress addr_2 = hebiMacAddressFromBytes(0x00, 0x1e, 0xc0, 0x8d, 0x79, 0xd8);
  macs[1] = &addr_2;

  const char* names[2];
  size_t num_names = sizeof(names) / sizeof(names[0]);
  char name_1[6] = "SA034";
  names[0] = name_1;
  char name_2[6] = "SA030";
  names[1] = name_2;

  const char* families[1];
  const size_t num_families = sizeof(families) / sizeof(families[0]);
  char fam_name_1[4] = "arm";
  families[0] = fam_name_1;

  // Test out lookup methods; use a 4 second timeout.
  long timeout_ms = 4000;

  printf("Looking up group by MAC addresses.\n");
  HebiGroupPtr group = hebiGroupCreateFromMacs(lookup, macs, num_macs, timeout_ms);
  checkAndReleaseGroup(group);

  printf("Looking up group by name.\n");
  group = hebiGroupCreateFromNames(lookup,
                                   families, num_families,
                                   names, num_names, timeout_ms);
  checkAndReleaseGroup(group);

  printf("Looking up group by family.\n");
  group = hebiGroupCreateFromFamily(lookup, families[0], timeout_ms);
  checkAndReleaseGroup(group);

  printf("Looking up connected group by mac.\n");
  group = hebiGroupCreateConnectedFromMac(lookup, macs[0], timeout_ms);
  checkAndReleaseGroup(group);

  printf("Looking up connected group by name.\n");
  group = hebiGroupCreateConnectedFromName(lookup, families[0], names[0], timeout_ms);
  checkAndReleaseGroup(group);

  // Always delete the lookup after you are done with it.
  hebiLookupRelease(lookup);
  hebiCleanup();

  return 0;
}
