/**
 * This file demonstrates the group lookup capabilities of the API. The lookup
 * allows you to create groups of modules based on which ones are available on
 * the network.
 */

#include <stdio.h>
#include "hebi.h"

////////////////////////////////////////////////////////////////////////////////
// Define 'sleep' as a cross-platform function
////////////////////////////////////////////////////////////////////////////////

#if !defined(_WIN32) && (defined(__unix__) || defined(__unix) || (defined(__APPLE__) && defined(__MACH__)))
#include <unistd.h>

static inline void hebi_sleep_s(unsigned int s)
{
  sleep(s);
}

#else
#include <Windows.h>

static inline void hebi_sleep_s(unsigned int s)
{
  Sleep(s * 1000);
}

#endif

////////////////////////////////////////////////////////////////////////////////
// These are kept as global variables for the purpose of example readability.
////////////////////////////////////////////////////////////////////////////////
HebiLookupPtr lookup = NULL;
HebiGroupPtr group = NULL;

////////////////////////////////////////////////////////////////////////////////
// This is #defined so that the size of the arrays below can be stack allocated;
// using a "const int" for this does not work in MSVC.
////////////////////////////////////////////////////////////////////////////////
#define NAME_FAMILY_BUFFER_LENGTH 100

////////////////////////////////////////////////////////////////////////////////
// Attempts to look for several groups on the network (hardcoded addresses below),
// and retrieve references to them.
////////////////////////////////////////////////////////////////////////////////
int example()
{
  // Create the lookup object
  lookup = hebiLookupCreate();
  if (lookup == NULL)
  {
    printf("createLookup() returned NULL.\n\n");
    return 1;
  }

  // Wait for the module list to populate, and print out its contents
  hebi_sleep_s(2);

  // Iterate through the items discovered by the lookup so far.
  HebiLookupEntryListPtr entry_list = hebiCreateLookupEntryList(lookup);
  int i = 0;
  int num_entries = hebiLookupEntryListGetSize(entry_list);
  // Buffers to store name/family of each item; this example only supports names
  // with less than 100 characters.
  char name[NAME_FAMILY_BUFFER_LENGTH];
  char family[NAME_FAMILY_BUFFER_LENGTH];
  for (i = 0; i < num_entries; ++i)
  {
    size_t buf_size = NAME_FAMILY_BUFFER_LENGTH;
    if (hebiLookupEntryListGetName(entry_list, i, name, &buf_size) == HebiStatusSuccess)
      printf("Name: %s\n", name);
    else
      printf("Name: too large for buffer\n");
    buf_size = NAME_FAMILY_BUFFER_LENGTH;
    if (hebiLookupEntryListGetFamily(entry_list, i, family, &buf_size) == HebiStatusSuccess)
      printf("Family: %s\n", family);
    else
      printf("Family: too large for buffer\n");
    HebiMacAddress addr;
    if (hebiLookupEntryListGetMacAddress(entry_list, i, &addr) == HebiStatusSuccess)
      printf("Mac Address: %02x:%02x:%02x:%02x:%02x:%02x\n",
        addr.bytes_[0], addr.bytes_[1], addr.bytes_[2], addr.bytes_[3], addr.bytes_[4], addr.bytes_[5]);
    else
      printf("Could not read MacAddress from entry\n");
    printf("\n");
  }
  hebiLookupEntryListRelease(entry_list);
  printf("\n");

  // Actually create the group (NOTE: change these values and the corresponding
  // buffer lengths when looking up different modules)
  char* names[3];
  int num_names = sizeof(names) / sizeof(names[0]);
  char name_1[5] = "base";
  names[0] = name_1;
  char name_2[9] = "shoulder";
  names[1] = name_2;
  char name_3[6] = "elbow";
  names[2] = name_3;

  char* families[1];
  const int num_families = sizeof(families) / sizeof(families[0]);
  char fam_name_1[7] = "family";
  families[0] = fam_name_1;

  // Test out lookup methods; use a 4 second timeout.
  long timeout_ms = 4000;

  printf("Looking up group by name.\n");
  group = hebiGroupCreateFromNames(lookup,
                                   (const char* const*) families, num_families,
                                   (const char* const*) names, num_names, timeout_ms);
  if (group == NULL)
  {
    printf("Group not found on network.\n\n");
    return 1;
  }
  
  printf("Found group with %zu modules.\n\n", hebiGroupGetSize(group));
  return 0;
}

////////////////////////////////////////////////////////////////////////////////
// Main file is responsible for cleanup here
////////////////////////////////////////////////////////////////////////////////
int main(int argc, char* argv[])
{
  // Run example:
  int res = example();

  // Cleanup:
  if (group != NULL)
    hebiGroupRelease(group);
  if (lookup != NULL)
    hebiLookupRelease(lookup);

  hebiCleanup();

  return res;
}
