/**
 * This file demonstrates the ability to get feedback from a group.
 */

#include <stdio.h>
#include <inttypes.h>

#include "hebi.h"

////////////////////////////////////////////////////////////////////////////////
// Define 'sleep' as a cross-platform function
////////////////////////////////////////////////////////////////////////////////

#if !defined(_WIN32) && (defined(__unix__) || defined(__unix) || (defined(__APPLE__) && defined(__MACH__)))
#include <unistd.h>

static inline void hebi_sleep_s(unsigned int s)
{
  sleep(s);
}

#else
#include <Windows.h>

static inline void hebi_sleep_s(unsigned int s)
{
  Sleep(s * 1000);
}

#endif

////////////////////////////////////////////////////////////////////////////////
// These are kept as global variables for the purpose of example readability.
////////////////////////////////////////////////////////////////////////////////
HebiLookupPtr lookup = NULL;
HebiGroupPtr group = NULL;
HebiGroupFeedbackPtr group_fbk = NULL;

////////////////////////////////////////////////////////////////////////////////
// Helper function for printing out feedback.
////////////////////////////////////////////////////////////////////////////////
void printFeedback(HebiFeedbackPtr feedback)
{
  if (hebiFeedbackGetHighResAngle(feedback, HebiFeedbackHighResAnglePosition, NULL, NULL) == HebiStatusSuccess)
  {
    int64_t revs; float offset;
    hebiFeedbackGetHighResAngle(feedback, HebiFeedbackHighResAnglePosition, &revs, &offset);
    printf("%" PRId64 " revolutions plus %f radians\n", revs, offset);
  }
  else
  {
    printf("no position feedback for this module!\n");
  }
}

////////////////////////////////////////////////////////////////////////////////
// This is our callback function for the async feedback loop:
// Note: the user data function can be used, for example, to store a reference
// to something in the main function.  Here, we use it to update a count of
// total messages. In C++ code, this could be a pointer to the class so this can
// effectively be treated like a member function.
////////////////////////////////////////////////////////////////////////////////
void feedbackCallback(HebiGroupFeedbackPtr feedback, void * user_data)
{
  // Print out position of first module:
  HebiFeedbackPtr first_module_feedback =
    hebiGroupFeedbackGetModuleFeedback(feedback, 0);
  printFeedback(first_module_feedback);

  // Increment feedback count:
  int* feedback_count = (int*)user_data;
  (*feedback_count)++;
}

////////////////////////////////////////////////////////////////////////////////
// Demonstrates getting feedback in syncronous and asyncronous manner
////////////////////////////////////////////////////////////////////////////////
int example()
{
  //////////////////////////////////////////////////////////////////////////////
  // Get the group
  //////////////////////////////////////////////////////////////////////////////
  lookup = hebiLookupCreate();
  if (lookup == NULL)
  {
    printf("createLookup() returned NULL.\n\n");
    return 1;
  }

  // Actually create the group (NOTE: change these values and the corresponding
  // buffer lengths when looking up different modules)
  char* names[3];
  int num_names = sizeof(names) / sizeof(names[0]);
  char name_1[5] = "base";
  names[0] = name_1;
  char name_2[9] = "shoulder";
  names[1] = name_2;
  char name_3[6] = "elbow";
  names[2] = name_3;

  char* families[1];
  const int num_families = sizeof(families) / sizeof(families[0]);
  char fam_name_1[7] = "family";
  families[0] = fam_name_1;

  // Test out lookup methods; use a 4 second timeout.
  long timeout_ms = 4000;

  group = hebiGroupCreateFromNames(lookup,
                                   (const char* const*) families, num_families,
                                   (const char* const*) names, num_names, timeout_ms);
  if (group == NULL)
  {
    printf("Group not found on network.\n\n");
    return 1;
  }

  //////////////////////////////////////////////////////////////////////////////
  // Get feedback - blocking call
  //////////////////////////////////////////////////////////////////////////////

  int num_modules = hebiGroupGetSize(group);
  group_fbk = hebiGroupFeedbackCreate(num_modules);
  if (group_fbk == NULL)
  {
    printf("Could not create group feedback object.\n\n");
    return 1;
  }

  // First send request (note -- this can be eliminated if you have a non-zero
  // feedback frequency)
  if (hebiGroupSendFeedbackRequest(group) != 0)
  {
    printf("Could not send feedback request.\n\n");
    return 1;
  }

  // Then get response
  if (hebiGroupGetNextFeedback(group, group_fbk, timeout_ms) != 0)
  {
    printf("Did not receive feedback within specified time.\n\n");
    return 1;
  }
  
  // Display feedback from first module:
  printf("Got feedback with positions: \n");
  printFeedback(hebiGroupFeedbackGetModuleFeedback(group_fbk, 0));

  //////////////////////////////////////////////////////////////////////////////
  // Get feedback - asyncronous callbacks
  //////////////////////////////////////////////////////////////////////////////
 
  // Note that the third parameter is a void* 'user_data' field that is passed
  // by the callbacks. This can be a complex data structure that you want to be
  // accessible from the callback; here we demonstrate with an integer. Remember
  // to use good multithreading practice regarding variable acces here, as the
  // callback will be from a different thread.
  int total_feedback_calls = 0;
  hebiGroupRegisterFeedbackHandler(group, feedbackCallback, &total_feedback_calls);

  // Start 200Hz feedback loop.
  printf("Starting asynchronous feedback callbacks\n");
  hebiGroupSetFeedbackFrequencyHz(group, 200);
  // Wait 10 seconds. This should result in about 2,000 callbacks.
  int wait_period_s = 10;
  hebi_sleep_s(wait_period_s);
  // Stop the feedback loop, and unrelease our callback:
  hebiGroupSetFeedbackFrequencyHz(group, 0);
  hebiGroupClearFeedbackHandlers(group);

  printf("Feedback callback thread stopped; %d callbacks were made.\n", total_feedback_calls);
  return 0;
}

////////////////////////////////////////////////////////////////////////////////
// Main file is responsible for cleanup here
////////////////////////////////////////////////////////////////////////////////
int main(int argc, char* argv[])
{
  // Run example:
  int res = example();

  // Cleanup:
  if (group_fbk != NULL)
    hebiGroupFeedbackRelease(group_fbk);
  if (group != NULL)
    hebiGroupRelease(group);
  if (lookup != NULL)
    hebiLookupRelease(lookup);

  hebiCleanup();

  return res;
}
